@@ -22,13 +22,23 @@ class BaseTrainer(AbstractTrainer):
             opacity_lr=0.025,
             scaling_lr=0.005,
             rotation_lr=0.001,
-            ignore_out_of_image_mask_loss=False,  # whether to ignore loss for out-of-mask pixels, if True, these pixels will be ignored in loss computation
-            random_out_of_image_mask_color=False,  # if ignore_out_of_mask_loss is False, whether use random color or use camera.bg_color for out-of-mask pixels
+            mask_mode="none",
+            # "none"=do not use mask
+            # "ignore"=loss of the masked area will be set to 0
+            # "bg_color"=fill the masked area of ground truth with the bg_color for rendering
+            bg_color=None,
+            # None=do not change bg_color
+            # "random"=set bg_color to random color
+            # tuple(float, float, float)=set bg_color to the given color
     ):
         super().__init__()
         self.lambda_dssim = lambda_dssim
-        self.ignore_out_of_image_mask_loss = ignore_out_of_image_mask_loss
-        self.random_out_of_image_mask_color = random_out_of_image_mask_color
+        assert mask_mode in ["none", "ignore", "bg_color"], f"Unknown mask policy: {mask_mode}"
+        assert bg_color is None or bg_color == "random" or (
+            isinstance(bg_color, tuple) and len(bg_color) == 3 and all(isinstance(c, float) for c in bg_color)
+        ), f"bg_color must be 'random' or a RGB value tuple(float, float, float), got {bg_color}"
+        self.mask_mode = mask_mode
+        self.bg_color = bg_color
         params = [
             {'params': [model._xyz], 'lr': position_lr_init * scene_extent, "name": "xyz"},
             {'params': [model._features_dc], 'lr': feature_lr, "name": "f_dc"},
@@ -71,18 +81,35 @@ class BaseTrainer(AbstractTrainer):
     def schedulers(self) -> Dict[str, Callable[[int], float]]:
         return self._schedulers
 
+    def preprocess(self, camera: Camera) -> Camera:
+        if self.bg_color == "random":
+            camera = camera._replace(bg_color=torch.rand_like(camera.bg_color))
+        elif isinstance(self.bg_color, tuple):
+            camera = camera._replace(bg_color=torch.tensor(self.bg_color, device=camera.bg_color.device, dtype=camera.bg_color.dtype))
+        elif self.bg_color is None:
+            pass
+        else:
+            raise ValueError(f"bg_color must be 'random' or a tuple(int, int, int), got {self.bg_color}")
+        return camera
+
     def loss(self, out: dict, camera: Camera) -> torch.Tensor:
         render = out["render"]
         gt = camera.ground_truth_image
         mask = camera.ground_truth_image_mask
-        if mask is not None:
-            if self.ignore_out_of_image_mask_loss:
+        match self.mask_mode:
+            case "none":
+                pass
+            case "ignore":
+                assert mask is not None, "Mask is required for 'ignore' mask policy"
                 render = render * mask.unsqueeze(0)
                 gt = gt * mask.unsqueeze(0)
-            elif self.random_out_of_image_mask_color:
-                gt = gt * mask.unsqueeze(0) + (1 - mask.unsqueeze(0)) * torch.rand_like(gt)
-            else:
-                gt = gt * mask.unsqueeze(0) + (1 - mask.unsqueeze(0)) * camera.bg_color.unsqueeze(-1).unsqueeze(-1)
+            case "bg_color":
+                assert mask is not None, "Mask is required for 'bg_color' mask policy"
+                # bg_color after postprocess
+                bg_color = camera.postprocess(camera, camera.bg_color.unsqueeze(-1).unsqueeze(-1)).clamp(0.0, 1.0)
+                gt = gt * mask.unsqueeze(0) + (1 - mask.unsqueeze(0)) * bg_color
+            case _:
+                raise ValueError(f"Unknown mask policy: {self.mask_mode}")
         Ll1 = l1_loss(render, gt)
         ssim_value = ssim(render, gt)
         loss = (1.0 - self.lambda_dssim) * Ll1 + self.lambda_dssim * (1.0 - ssim_value)
